pub struct BigEnumSet<T: BigEnumSetType> { /* private fields */ }Expand description
An efficient set type for enums.
It is implemented using a bitset stored as [usize; N], where N is the smallest number that
such that the array can fit all the bits of the underlying enum. An enum with discriminant n
is stored in the n / WORD_SIZE word at the n % WORD_SIZE least significant bit (corresponding
with as bit mask of 1 << (n % WORD_SIZE). WORD_SIZE is mem::size_of::<usize>().
Serialization
When the serde feature is enabled, BigEnumSets can be serialized and deserialized using
the serde crate. The exact serialization format can be controlled
with additional attributes on the enum type. These attributes are valid regardless of
whether the serde feature is enabled.
By default BigEnumSets are serialized as [u8; N], where N is smallest such that the array
can fit all bits that are part of the underlying enum. An enum with discriminant n is serialized
as n % 8th least significant bit in the n / 8 byte. You can add a
#[big_enum_set(serialize_bytes = N)] attribute to your enum to control the number of bytes
in the serialization. This can be important for avoiding unintentional breaking changes when
BigEnumSets are serialized with formats like bincode.
By default, unknown bits are ignored and silently removed from the bitset. To override this
behavior, you can add a #[big_enum_set(serialize_deny_unknown)] attribute. This will cause
deserialization to fail if an invalid bit is set.
In addition, the #[big_enum_set(serialize_as_list)] attribute causes the BigEnumSet to be
instead serialized as a list of enum variants. This requires your enum type implement
Serialize and Deserialize. Note that this is a breaking change.
Implementations
Sourceimpl<T: BigEnumSetType> BigEnumSet<T>
impl<T: BigEnumSetType> BigEnumSet<T>
Sourcepub const EMPTY: BigEnumSet<T>
pub const EMPTY: BigEnumSet<T>
Empty set.
Sourcepub fn empty() -> Self
pub fn empty() -> Self
Creates an empty BigEnumSet.
This is an alias for BigEnumSet::new.
Sourcepub fn bit_width() -> u32
pub fn bit_width() -> u32
Total number of bits used by this type. Note that the actual amount of space used is
rounded up to the next highest usize.
This is the same as BigEnumSet::variant_count except in enums with “sparse” variants.
(e.g. enum Foo { A = 10, B = 20 })
Sourcepub fn variant_count() -> u32
pub fn variant_count() -> u32
The number of valid variants this type may contain.
This is the same as BigEnumSet::bit_width except in enums with “sparse” variants.
(e.g. enum Foo { A = 10, B = 20 })
Sourcepub fn try_from_bits(bits: &[usize]) -> Option<Self>
pub fn try_from_bits(bits: &[usize]) -> Option<Self>
Constructs a BigEnumSet from raw bits.
Returns None if there are any invalid bits set in bits.
The size of bits need not match the underlying representation.
Sourcepub fn from_bits_truncated(bits: &[usize]) -> Self
pub fn from_bits_truncated(bits: &[usize]) -> Self
Constructs a BigEnumSet from raw bits, ignoring any unknown variants.
The size of bits need not match the underlying representation.
Sourcepub fn is_disjoint<O: Borrow<Self>>(&self, other: O) -> bool
pub fn is_disjoint<O: Borrow<Self>>(&self, other: O) -> bool
Returns true if self has no elements in common with other. This is equivalent to
checking for an empty intersection.
Sourcepub fn is_superset<O: Borrow<Self>>(&self, other: O) -> bool
pub fn is_superset<O: Borrow<Self>>(&self, other: O) -> bool
Returns true if self is a superset of other, i.e., self contains at least all the
elements in other.
Sourcepub fn is_subset<O: Borrow<Self>>(&self, other: O) -> bool
pub fn is_subset<O: Borrow<Self>>(&self, other: O) -> bool
Returns true if self is a subset of other, i.e., other contains at least all
the elements in self.
Sourcepub fn union<O: Borrow<Self>>(&self, other: O) -> Self
pub fn union<O: Borrow<Self>>(&self, other: O) -> Self
Returns a set containing all elements present in either set.
Sourcepub fn intersection<O: Borrow<Self>>(&self, other: O) -> Self
pub fn intersection<O: Borrow<Self>>(&self, other: O) -> Self
Returns a set containing all elements present in both sets.
Sourcepub fn difference<O: Borrow<Self>>(&self, other: O) -> Self
pub fn difference<O: Borrow<Self>>(&self, other: O) -> Self
Returns a set containing all elements present in self but not in other.
Sourcepub fn symmetrical_difference<O: Borrow<Self>>(&self, other: O) -> Self
pub fn symmetrical_difference<O: Borrow<Self>>(&self, other: O) -> Self
Returns a set containing all elements present in either self or other, but is not present
in both.
Sourcepub fn complement(&self) -> Self
pub fn complement(&self) -> Self
Returns a set containing all enum variants not present in this set.
Sourcepub fn insert(&mut self, value: T) -> bool
pub fn insert(&mut self, value: T) -> bool
Adds a value to this set.
If the set did not have this value present, true is returned.
If the set did have this value present, false is returned.
Sourcepub fn remove(&mut self, value: T) -> bool
pub fn remove(&mut self, value: T) -> bool
Removes a value from this set. Returns whether the value was present in the set.
Sourcepub fn insert_all<O: Borrow<Self>>(&mut self, other: O)
pub fn insert_all<O: Borrow<Self>>(&mut self, other: O)
Adds all elements in another set to this one.
Sourcepub fn remove_all<O: Borrow<Self>>(&mut self, other: O)
pub fn remove_all<O: Borrow<Self>>(&mut self, other: O)
Removes all values in another set from this one.
Sourcepub fn iter(&self) -> EnumSetIter<&BigEnumSet<T>, T> ⓘ
pub fn iter(&self) -> EnumSetIter<&BigEnumSet<T>, T> ⓘ
Creates an iterator over the values in this set.